#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['image_lite', 'image_lite.migrations', 'image_lite.templatetags']

package_data = \
{'': ['*'],
 'image_lite': ['management/*',
                'management/commands/*',
                'templates/*',
                'templates/image/*']}

setup(name='django_image_lite',
      version='0.0.1',
      description='Local image handling for Django. Unobtusive, with multiple repositories, powerful filter system and scaleable data.',
      author=None,
      author_email='robert crowther <rw.crowther@gmail.com>',
      url=None,
      packages=packages,
      package_data=package_data,
     )
